<?php

namespace Drupal\symfony_mailer_from_filter\Plugin\EmailAdjuster;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\symfony_mailer\Address;
use Drupal\symfony_mailer\EmailInterface;
use Drupal\symfony_mailer\Processor\EmailAdjusterBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the 'From filter' adjuster.
 *
 * @EmailAdjuster(
 *   id = "symfony_mailer_from_filter",
 *   label = @Translation("From filter"),
 *   description = @Translation("Limits all outgoing email 'From' addresses to a whitelist and moves unrecognized 'From' addresses to the 'Reply-to' address."),
 * )
 */
class SymfonyMailerFromFilterAdjuster extends EmailAdjusterBase implements ContainerFactoryPluginInterface {

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new SymfonyMailerFromFilterAdjuster object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin ID for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   */
  public function __construct(array $configuration, string $plugin_id, mixed $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory'),
    );
  }

  /**
   * The default weight for from filter to run last.
   */
  const int DEFAULT_WEIGHT = 600;

  /**
   * {@inheritdoc}
   */
  public function settingsForm(array $form, FormStateInterface $form_state): array {
    $form['from_allowed'] = [
      '#type' => 'textarea',
      '#title' => $this->t("Allowed 'From' email addresses"),
      '#description' => $this->t("Enter email addresses and domains that are allowed to be used via the 'From' address of all outgoing emails.")
        . '<br/>'
        . $this->t('Enter one email address per line.')
        . '<br/>'
        . $this->t('Wildcards are supported (e.g. *@example.com).')
        . '<br/>'
        . $this->t('Leave blank to filter all outgoing email.'),
      '#default_value' => $this->configuration['from_allowed'] ?? '',
    ];
    $form['from_display_name_preserve'] = [
      '#type' => 'checkbox',
      '#title' => $this->t("Preserve the 'From' display name"),
      '#default_value' => $this->configuration['from_display_name_preserve'] ?? FALSE,
      '#return_value' => TRUE,
      '#description' => $this->t("If checked and the 'From' display name is set, the 'From' display name value will not be replaced with the default 'From' display name entered below."),
    ];

    $form['from_default_email'] = [
      '#type' => 'email',
      '#title' => $this->t("Default 'From' email address"),
      '#default_value' => $this->configuration['from_default_email'] ?? NULL,
      '#description' => $this->t("Enter the email address to be used as 'From' email address when the outgoing email's 'From' email address is not allowed.")
        . '<br/>'
        . $this->t("If left blank, the site's email address be used as the default 'From' email address."),
    ];
    $form['from_default_display_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t("Default 'From' display name"),
      '#default_value' => $this->configuration['from_default_display_name'] ?? NULL,
      '#description' => $this->t("Enter the display name to be used as 'From' display name when the outgoing email's 'From' email address is not allowed.")
        . '<br/>'
        . $this->t("If left blank, the site's name will be used as the default 'From' display name."),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function postRender(EmailInterface $email): void {
    // Get from address.
    $from = $email->getFrom();
    $from_email = (isset($from[0])) ? $from[0]->getEmail() : '';
    $from_display_name = (isset($from[0])) ? $from[0]->getDisplayName() : '';
    // Don't do any filtering is the 'From' address is empty.
    if (empty($from_email)) {
      return;
    }

    // Get allowed from addresses.
    $from_allowed = $this->configuration['from_allowed'] ?? '';

    // Get allowed email patterns with wild card support.
    $patterns = preg_split("/(\r\n|\n|\r)/", str_replace('*', '.*', $from_allowed));
    $patterns = array_map('trim', $patterns);
    $pattern = '/^(' . implode('|', $patterns) . ')$/';

    // Move 'From' address to 'Reply-to' address if the allowed email addresses
    // is blank or doesn't match the 'From' address.
    if (empty($from_allowed) || !preg_match($pattern, $from_email)) {
      // Get from default email and display name.
      $from_default_email = $this->configuration['from_default_email']
        ?: $this->configFactory->get('system.site')->get('mail');
      $from_default_display_name = $this->configuration['from_default_display_name']
        ?: $this->configFactory->get('system.site')->get('name');

      // Preserve the 'From' display name.
      $from_display_name_preserve = $this->configuration['from_display_name_preserve'];
      if ($from_display_name_preserve && $from_display_name) {
        $from_default_display_name = $from_display_name;
      }

      $email->setFrom(new Address($from_default_email, $from_default_display_name));
      $email->setReplyTo($from);
    }
  }

}
