<?php

namespace Drupal\Tests\symfony_mailer_from_filer\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\symfony_mailer\Address;
use Drupal\symfony_mailer\EmailFactoryInterface;
use Drupal\symfony_mailer\EmailInterface;
use Drupal\symfony_mailer\Processor\EmailAdjusterManagerInterface;
use Drupal\symfony_mailer_from_filter\Plugin\EmailAdjuster\SymfonyMailerFromFilterAdjuster;

/**
 * Tests basic email sending.
 *
 * @group filter
 */
class SymfonyMailerFromFilterAdjusterKernelTest extends KernelTestBase {

  /**
   * Email address for the tests.
   */
  protected string $addressTo = 'symmfony-mailer-to@example.com';

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'symfony_mailer',
    'symfony_mailer_from_filter',
    'system',
  ];

  /**
   * The email factory.
   */
  protected EmailFactoryInterface $emailFactory;

  /**
   * The email adjuster plugin manager.
   */
  protected EmailAdjusterManagerInterface $emailAdjusterManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->emailFactory = $this->container->get('email_factory');
    $this->emailAdjusterManager = $this->container->get('plugin.manager.email_adjuster');

    $this->config('system.site')
      ->set('name', 'Site')
      ->set('mail', 'administrator@site.com')
      ->save();
  }

  /**
   * Basic email sending test.
   */
  public function testEmail(): void {
    // Check with no configuration and not 'From' address nothing is set.
    $configuration = [
      'from_allowed' => '',
      'from_display_name_preserve' => FALSE,
      'from_default_email' => '',
      'from_default_display_name' => '',
    ];
    $from_filter_adjuster = $this->getFromFilterAdjuster($configuration);
    $email = $this->emailFactory->newTypedEmail('symfony_mailer', 'test', $this->addressTo);
    $from_filter_adjuster->postRender($email);
    $this->assertEmailFrom($email, '', '');
    $this->assertEmailReplayTo($email, '', '');

    // Check with 'From' address the 'Reply-to' address is set to the 'From' address.
    $configuration = [
      'from_allowed' => '',
      'from_display_name_preserve' => FALSE,
      'from_default_email' => '',
      'from_default_display_name' => '',
    ];
    $from_filter_adjuster = $this->getFromFilterAdjuster($configuration);
    $email = $this->emailFactory->newTypedEmail('symfony_mailer', 'test', $this->addressTo);
    $email->setFrom(new Address('from@example.com', 'From'));
    $from_filter_adjuster->postRender($email);
    $this->assertEmailFrom($email, 'administrator@site.com', 'Site');
    $this->assertEmailReplayTo($email, 'from@example.com', 'From');

    // Check that the from default email and display name is used as expected.
    $configuration = [
      'from_allowed' => '',
      'from_display_name_preserve' => FALSE,
      'from_default_email' => 'no-reply@example.com',
      'from_default_display_name' => 'No reply',
    ];
    $from_filter_adjuster = $this->getFromFilterAdjuster($configuration);
    $email = $this->emailFactory->newTypedEmail('symfony_mailer', 'test', $this->addressTo);
    $email->setFrom(new Address('from@example.com', 'From'));
    $from_filter_adjuster->postRender($email);
    $this->assertEmailFrom($email, 'no-reply@example.com', 'No reply');
    $this->assertEmailReplayTo($email, 'from@example.com', 'From');

    // Check that the from display name is preserved as expected..
    $configuration = [
      'from_allowed' => '',
      'from_display_name_preserve' => TRUE,
      'from_default_email' => 'no-reply@example.com',
      'from_default_display_name' => 'No reply',
    ];
    $from_filter_adjuster = $this->getFromFilterAdjuster($configuration);
    $email = $this->emailFactory->newTypedEmail('symfony_mailer', 'test', $this->addressTo);
    $email->setFrom(new Address('from@example.com', 'From'));
    $from_filter_adjuster->postRender($email);
    $this->assertEmailFrom($email, 'no-reply@example.com', 'From');
    $this->assertEmailReplayTo($email, 'from@example.com', 'From');

    // Check with allowed 'From' address the 'Reply-to' address is not set.
    $configuration = [
      'from_allowed' => '*@example.com
other@other.com',
      'from_display_name_preserve' => FALSE,
      'from_default_email' => 'no-reply@example.com',
      'from_default_display_name' => 'No reply',
    ];
    $from_filter_adjuster = $this->getFromFilterAdjuster($configuration);
    $email = $this->emailFactory->newTypedEmail('symfony_mailer', 'test', $this->addressTo);
    $email->setFrom(new Address('from@example.com', 'From'));
    $from_filter_adjuster->postRender($email);
    $this->assertEmailFrom($email, 'from@example.com', 'From');
    $this->assertEmailReplayTo($email, '', '');
  }

  /**
   * Asserts that the "from" address of the given email matches the expected email and display name.
   *
   * @param \Drupal\symfony_mailer\EmailInterface $email
   *   The email object to verify.
   * @param string $from_email
   *   The expected "from" email address.
   * @param string $from_display_name
   *   The expected "from" display name.
   */
  protected function assertEmailFrom(EmailInterface $email, string $from_email, string $from_display_name): void {
    $from = $email->getFrom();
    $this->assertEquals($from_email, $from ? $from[0]->getEmail() : '');
    $this->assertEquals($from_display_name, $from ? $from[0]->getDisplayName() : '');
  }

  /**
   * Asserts that the email's reply-to address and display name match the expected values.
   *
   * @param \Drupal\symfony_mailer\EmailInterface $email
   *   The email object to be tested.
   * @param string $reply_to_email
   *   The expected reply-to email address.
   * @param string $reply_to_display_name
   *   The expected reply-to display name.
   */
  protected function assertEmailReplayTo(EmailInterface $email, string $reply_to_email, string $reply_to_display_name): void {
    $reply_to = $email->getReplyTo();
    $this->assertEquals($reply_to_email, $reply_to ? $reply_to[0]->getEmail() : '');
    $this->assertEquals($reply_to_display_name, $reply_to ? $reply_to[0]->getDisplayName() : '');
  }

  /**
   * Creates and returns an instance of SymfonyMailerFromFilterAdjuster using the provided configuration.
   *
   * @param string[] $configuration
   *   An optional associative array of configuration values. The following keys
   *   can be provided:
   *   - 'from_allowed': A string defining allowed "from" addresses.
   *   - 'from_default_email': A string specifying the default "from" email address.
   *   - 'from_default_display_name': A string representing the default "from" display name.
   *
   * @return \Drupal\symfony_mailer_from_filter\Plugin\EmailAdjuster\SymfonyMailerFromFilterAdjuster
   *   A configured instance of SymfonyMailerFromFilterAdjuster.
   */
  protected function getFromFilterAdjuster(array $configuration = []): SymfonyMailerFromFilterAdjuster {
    $configuration += [
      'from_allowed' => '',
      'from_display_name_preserve' => FALSE,
      'from_default_email' => '',
      'from_default_display_name' => '',
    ];
    $definition = $this->emailAdjusterManager->getDefinition('symfony_mailer_from_filter');
    return SymfonyMailerFromFilterAdjuster::create(
      $this->container,
      $configuration,
      'symfony_mailer_from_filter',
      $definition
    );
  }

}
